"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toBeChecked = toBeChecked;
exports.toBeEditable = toBeEditable;
exports.toBeEnabled = toBeEnabled;
exports.toBeDisabled = toBeDisabled;
exports.toBeEmpty = toBeEmpty;
exports.toBeHidden = toBeHidden;
exports.toBeVisible = toBeVisible;
exports.toBeFocused = toBeFocused;
exports.toBeSelected = toBeSelected;

var _jestMatcherUtils = require("jest-matcher-utils");

var _globals = require("../globals");

var _util = require("../util");

/**
 * Copyright Microsoft Corporation. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
async function toBeTruthyImpl(matcherName, locator, query, options = {}) {
  const testInfo = (0, _globals.currentTestInfo)();
  if (!testInfo) throw new Error(`${matcherName} must be called during the test`);
  (0, _util.expectLocator)(locator, matcherName);
  const matcherOptions = {
    isNot: this.isNot,
    promise: this.promise
  };
  let received;
  let pass = false;
  const timeout = options.timeout === 0 ? 0 : options.timeout || testInfo.timeout;
  const deadline = timeout ? (0, _util.monotonicTime)() + timeout : 0; // TODO: interrupt on timeout for nice message.

  await (0, _util.pollUntilDeadline)(async () => {
    const remainingTime = deadline ? deadline - (0, _util.monotonicTime)() : 0;
    received = await query(remainingTime);
    pass = !!received;
    return pass === !matcherOptions.isNot;
  }, deadline, 100);

  const message = () => {
    return (0, _jestMatcherUtils.matcherHint)(matcherName, undefined, '', matcherOptions);
  };

  return {
    message,
    pass
  };
}

async function toBeChecked(locator, options) {
  return toBeTruthyImpl.call(this, 'toBeChecked', locator, async timeout => {
    return await locator.isChecked({
      timeout
    });
  }, options);
}

async function toBeEditable(locator, options) {
  return toBeTruthyImpl.call(this, 'toBeEditable', locator, async timeout => {
    return await locator.isEditable({
      timeout
    });
  }, options);
}

async function toBeEnabled(locator, options) {
  return toBeTruthyImpl.call(this, 'toBeEnabled', locator, async timeout => {
    return await locator.isEnabled({
      timeout
    });
  }, options);
}

async function toBeDisabled(locator, options) {
  return toBeTruthyImpl.call(this, 'toBeDisabled', locator, async timeout => {
    return await locator.isDisabled({
      timeout
    });
  }, options);
}

async function toBeEmpty(locator, options) {
  return toBeTruthyImpl.call(this, 'toBeEmpty', locator, async timeout => {
    return await locator.evaluate(element => {
      var _element$textContent;

      if (element.nodeName === 'INPUT' || element.nodeName === 'TEXTAREA') return !element.value;
      return !((_element$textContent = element.textContent) !== null && _element$textContent !== void 0 && _element$textContent.trim());
    }, {
      timeout
    });
  }, options);
}

async function toBeHidden(locator, options) {
  return toBeTruthyImpl.call(this, 'toBeHidden', locator, async timeout => {
    return await locator.isHidden({
      timeout
    });
  }, options);
}

async function toBeVisible(locator, options) {
  return toBeTruthyImpl.call(this, 'toBeVisible', locator, async timeout => {
    return await locator.isVisible({
      timeout
    });
  }, options);
}

async function toBeFocused(locator, options) {
  return toBeTruthyImpl.call(this, 'toBeFocused', locator, async timeout => {
    return await locator.evaluate(element => {
      return document.activeElement === element;
    }, {
      timeout
    });
  }, options);
}

async function toBeSelected(locator, options) {
  return toBeTruthyImpl.call(this, 'toBeSelected', locator, async timeout => {
    return await locator.evaluate(element => {
      return element.selected;
    }, {
      timeout
    });
  }, options);
}