"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toHaveText = toHaveText;
exports.toContainText = toContainText;
exports.toHaveAttr = toHaveAttr;
exports.toHaveData = toHaveData;
exports.toHaveCSS = toHaveCSS;
exports.toHaveId = toHaveId;
exports.toHaveValue = toHaveValue;
exports.toHaveClass = toHaveClass;

var _print = require("expect/build/print");

var _jestMatcherUtils = require("jest-matcher-utils");

var _globals = require("../globals");

var _util = require("../util");

/**
 * Copyright Microsoft Corporation. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
async function toMatchTextImpl(matcherName, locator, query, expected, options = {}) {
  const testInfo = (0, _globals.currentTestInfo)();
  if (!testInfo) throw new Error(`${matcherName} must be called during the test`);
  (0, _util.expectLocator)(locator, matcherName);
  const matcherOptions = {
    isNot: this.isNot,
    promise: this.promise
  };

  if (!(typeof expected === 'string') && !(expected && typeof expected.test === 'function')) {
    throw new Error((0, _jestMatcherUtils.matcherErrorMessage)((0, _jestMatcherUtils.matcherHint)(matcherName, undefined, undefined, matcherOptions), `${(0, _jestMatcherUtils.EXPECTED_COLOR)('expected')} value must be a string or regular expression`, (0, _jestMatcherUtils.printWithType)('Expected', expected, _jestMatcherUtils.printExpected)));
  }

  let received;
  let pass = false;
  const timeout = options.timeout === 0 ? 0 : options.timeout || testInfo.timeout;
  const deadline = timeout ? (0, _util.monotonicTime)() + timeout : 0; // TODO: interrupt on timeout for nice message.

  await (0, _util.pollUntilDeadline)(async () => {
    const remainingTime = deadline ? deadline - (0, _util.monotonicTime)() : 0;
    received = await query(remainingTime);
    if (options.matchSubstring) pass = received.includes(expected);else if (typeof expected === 'string') pass = received === expected;else pass = expected.test(received);
    return pass === !matcherOptions.isNot;
  }, deadline, 100);
  const stringSubstring = options.matchSubstring ? 'substring' : 'string';
  const message = pass ? () => typeof expected === 'string' ? (0, _jestMatcherUtils.matcherHint)(matcherName, undefined, undefined, matcherOptions) + '\n\n' + `Expected ${stringSubstring}: not ${(0, _jestMatcherUtils.printExpected)(expected)}\n` + `Received string:        ${(0, _print.printReceivedStringContainExpectedSubstring)(received, received.indexOf(expected), expected.length)}` : (0, _jestMatcherUtils.matcherHint)(matcherName, undefined, undefined, matcherOptions) + '\n\n' + `Expected pattern: not ${(0, _jestMatcherUtils.printExpected)(expected)}\n` + `Received string:      ${(0, _print.printReceivedStringContainExpectedResult)(received, typeof expected.exec === 'function' ? expected.exec(received) : null)}` : () => {
    const labelExpected = `Expected ${typeof expected === 'string' ? stringSubstring : 'pattern'}`;
    const labelReceived = 'Received string';
    const printLabel = (0, _jestMatcherUtils.getLabelPrinter)(labelExpected, labelReceived);
    return (0, _jestMatcherUtils.matcherHint)(matcherName, undefined, undefined, matcherOptions) + '\n\n' + `${printLabel(labelExpected)}${(0, _jestMatcherUtils.printExpected)(expected)}\n` + `${printLabel(labelReceived)}${(0, _jestMatcherUtils.printReceived)(received)}`;
  };
  return {
    message,
    pass
  };
}

async function toHaveText(locator, expected, options) {
  return toMatchTextImpl.call(this, 'toHaveText', locator, async timeout => {
    if (options !== null && options !== void 0 && options.useInnerText) return await locator.innerText({
      timeout
    });
    return (await locator.textContent()) || '';
  }, expected, options);
}

async function toContainText(locator, expected, options) {
  return toMatchTextImpl.call(this, 'toContainText', locator, async timeout => {
    if (options !== null && options !== void 0 && options.useInnerText) return await locator.innerText({
      timeout
    });
    return (await locator.textContent()) || '';
  }, expected, { ...options,
    matchSubstring: true
  });
}

async function toHaveAttr(locator, name, expected, options) {
  return toMatchTextImpl.call(this, 'toHaveAttr', locator, async timeout => {
    return (await locator.getAttribute(name, {
      timeout
    })) || '';
  }, expected, options);
}

async function toHaveData(locator, name, expected, options) {
  return toMatchTextImpl.call(this, 'toHaveData', locator, async timeout => {
    return (await locator.getAttribute('data-' + name, {
      timeout
    })) || '';
  }, expected, options);
}

async function toHaveCSS(locator, name, expected, options) {
  return toMatchTextImpl.call(this, 'toHaveCSS', locator, async timeout => {
    return await locator.evaluate(async (element, name) => {
      return window.getComputedStyle(element)[name];
    }, name, {
      timeout
    });
  }, expected, options);
}

async function toHaveId(locator, expected, options) {
  return toMatchTextImpl.call(this, 'toHaveId', locator, async timeout => {
    return (await locator.getAttribute('id', {
      timeout
    })) || '';
  }, expected, options);
}

async function toHaveValue(locator, expected, options) {
  return toMatchTextImpl.call(this, 'toHaveValue', locator, async timeout => {
    return await locator.inputValue({
      timeout
    });
  }, expected, options);
}

async function toHaveClass(locator, expected, options) {
  return toMatchTextImpl.call(this, 'toHaveClass', locator, async timeout => {
    return await locator.evaluate(element => element.className, {
      timeout
    });
  }, expected, { ...options,
    matchSubstring: true
  });
}